from .raw import GiftRaw
from typing import *
import asyncio, logging

logger = logging.getLogger(__name__)

class Gift:
    def __init__(self, proxy: Optional[str] = None, batch_size: int = 1, delay: float = 0, impersonate: str = "safari_ios"):
        self.api = GiftRaw(proxy=proxy, batch_size=batch_size, delay=delay, impersonate=impersonate)


    async def floorPrice(self, name: Union[str, List[str]]):
        try:
            data = await self.api.CollectionInfo(name)
            
            if isinstance(name, list):
                results = []
                for item in data:
                    if isinstance(item, dict):
                        results.append(item.get("floorPrice", False))
                    else:
                        results.append(False)
                return results
            else:
                if isinstance(data, dict):
                    return data.get("floorPrice", False)
                return False
        except Exception as e:
            logger.error(f"[Gift]: Error in method floorPrice(). Error: {e}")
            return [] if isinstance(name, list) else False
        

    async def simpleEstimation(self, slug: Union[str, List[str]]):
        try:
            data = await self.api.GiftInfo(slug)
            
            if isinstance(slug, list):
                results = []
                for item in data:
                    if isinstance(item, dict):
                        results.append(item.get("priceAnalysisData", False).get("simpleEstimation").get("price"))
                    else:
                        results.append(False)
                return results
            else:
                if isinstance(data, dict):
                    return data.get("priceAnalysisData", False).get("simpleEstimation").get("price")
                return False
        except Exception as e:
            logger.error(f"[Gift]: Error in method simpleEstimation(). Error: {e}")
            return [] if isinstance(slug, list) else False
        

    async def estimatedPrice(self, slug: Union[str, List[str]], asset: Literal["Ton", "Usd"]="Ton"):
        try:
            data = await self.api.GiftInfo(slug)
            
            if isinstance(slug, list):
                results = []
                for item in data:
                    if isinstance(item, dict):
                        results.append(item.get(f"estimatedPrice{asset}", False))
                    else:
                        results.append(False)
                return results
            else:
                if isinstance(data, dict):
                    return data.get(f"estimatedPrice{asset}", False)
                return False
        except Exception as e:
            logger.error(f"[Gift]: Error in method estimatedPrice(). Error: {e}")
            return [] if isinstance(slug, list) else False
            

    async def models_floor(self, name: Union[str, List[str]]):
        try:
            if isinstance(name, list):
                tasks = []
                for item in name:
                    tasks.append(self._one_item(item, "Model"))
                
                results = await asyncio.gather(*tasks, return_exceptions=True)
                
                return {
                    item: result if not isinstance(result, Exception) else {}
                    for item, result in zip(name, results)
                }
            else:
                return await self._one_item(name, "Model")
        except Exception as e:
            logger.error(f"[Gift]: Error in method models_floor(). Error: {e}")
            return {} if isinstance(name, str) else {}
        

    async def backdrops_floor(self, name: Union[str, List[str]]):
        try:
            if isinstance(name, list):
                tasks = []
                for item in name:
                    tasks.append(self._one_item(item, "Backdrop"))
                
                results = await asyncio.gather(*tasks, return_exceptions=True)
                
                return {
                    item: result if not isinstance(result, Exception) else {}
                    for item, result in zip(name, results)
                }
            else:
                return await self._one_item(name, "Backdrop")
        except Exception as e:
            logger.error(f"[Gift]: Error in method backdrops_floor(). Error: {e}")
            return {} if isinstance(name, str) else {}
    

    async def symbols_floor(self, name: Union[str, List[str]]):
        try:
            if isinstance(name, list):
                tasks = []
                for item in name:
                    tasks.append(self._one_item(item, "Symbol"))
                
                results = await asyncio.gather(*tasks, return_exceptions=True)
                
                return {
                    item: result if not isinstance(result, Exception) else {}
                    for item, result in zip(name, results)
                }
            else:
                return await self._one_item(name, "Symbol")
        except Exception as e:
            logger.error(f"[Gift]: Error in method symbols_floor(). Error: {e}")
            return {} if isinstance(name, str) else {}
        

    async def _one_item(self, name: str, type: Literal["Model", "Backdrop", "Symbol"], key=""):
        try:
            formatted = name.replace(" ", "").replace("'", "").replace("-", "")
            url = f"https://app-api.xgift.tg/gifts/filters/{formatted}"

            response = await self.api._make_request(url=url, params={"collectionType": "upgradable"})
            
            if response is None:
                return {}
            
            result = {}
            try:
                data = response.json()
                f_name = "pattern" if type == "Symbol" else type.lower()
                for m in data.get(f"gift{type}", []):
                    if (n := m.get(f_name)) and (p := m.get("floorPriceTon")):
                        try:
                            result[n] = float(p)
                        except ValueError as e:
                            logger.error(f"[Gift]: ValueError in method _one_item(). Failed to convert price to float: {p}, Error: {e}")
                            result[n] = p
            except Exception as e:
                logger.error(f"[Gift]: Error in method _one_item(). Failed to parse JSON, Error: {e}")
                return {}
            
            return result
        except Exception as e:
            logger.error(f"[Gift]: Error in method _one_item(). Error: {e}")
            return {}

    async def getFloorGraph(self, slug: Union[str, List[str]]):
        try:
            data = await self.api.CollectionInfo(slug)
            
            if isinstance(slug, list):
                results = []
                for item in data:
                    if isinstance(item, dict):
                        results.append(item.get("floor"))
                    else:
                        results.append(False)
                return results
            else:
                if isinstance(data, dict):
                    return data.get("floor")
                return False
        except Exception as e:
            logger.error(f"[Gift]: Error in method getFloorGraph(). Error: {e}")
            return [] if isinstance(slug, list) else False
        

    async def isMonochrome(self, slug: Union[str, List[str]]):
        try:
            data = await self.api.GiftInfo(slug)
            
            if isinstance(slug, list):
                results = []
                for item in data:
                    if isinstance(item, dict):
                        results.append(item.get("isMonochrome", False))
                    else:
                        results.append(False)
                return results
            else:
                if isinstance(data, dict):
                    return data.get("isMonochrome", False)
                return False
        except Exception as e:
            logger.error(f"[Gift]: Error in method isMonochrome(). Error: {e}")
            return [] if isinstance(slug, list) else False
    

    async def close(self):
        try:
            await self.api.close()
        except Exception as e:
            logger.error(f"[Gift]: Error in method close(). Error: {e}")


import asyncio, logging
from typing import *
from curl_cffi.requests import AsyncSession, Response

logger = logging.getLogger(__name__)

class GiftRaw:
    def __init__(self, proxy: Optional[str] = None, batch_size: int = 1, delay: float = 0, impersonate: str = "safari_ios"):
        self.batch_size = batch_size
        self.delay = delay
        
        if proxy:
            self.client = AsyncSession(impersonate=impersonate, proxies={"http": proxy, "https": proxy})
        else:
            self.client = AsyncSession(impersonate=impersonate)
    

    async def _make_request(self, url: str, method: str = "GET", params: Optional[Dict] = None) -> Optional[Response]:
        try:
            response = await self.client.request(method=method, url=url, params=params, headers={'Accept': 'application/json, text/plain, */*', 'Accept-Language': 'en-US,en;q=0.9', 'Referer': 'https://xgift.tg/', 'Origin': 'https://xgift.tg', 'Sec-Fetch-Dest': 'empty', 'Sec-Fetch-Mode': 'cors', 'Sec-Fetch-Site': 'same-site'})
            response.raise_for_status()
            return response
        except Exception as e:
            logger.error(f"[GiftRaw]: Error in method _make_request(). URL: {url}, Error: {e}")
            return None
    

    async def GiftInfo(self, slug: Union[str, List[str]]):
        try:
            if isinstance(slug, list):
                results = []
                
                for i in range(0, len(slug), self.batch_size):
                    batch = slug[i:i + self.batch_size]
                    
                    tasks = []
                    for item in batch:
                        tasks.append(self._make_request(url=f"https://app-api.xgift.tg/gifts/{item}"))
                    
                    batch_responses = await asyncio.gather(*tasks, return_exceptions=True)
                    
                    for response in batch_responses:
                        if isinstance(response, Exception) or response is None:
                            results.append({})
                        else:
                            try:
                                results.append(response.json())
                            except Exception as e:
                                logger.error(f"[GiftRaw]: Error in method GiftInfo(). Failed to parse JSON, Error: {e}")
                                results.append({})
                    
                    if i + self.batch_size < len(slug) and self.delay > 0:
                        await asyncio.sleep(self.delay)
                
                return results

            else:
                response = await self._make_request(url=f"https://app-api.xgift.tg/gifts/{slug}")
                
                if response is None:
                    return {}
                
                try:
                    return response.json()
                except Exception as e:
                    logger.error(f"[GiftRaw]: Error in method GiftInfo(). Failed to parse JSON, Error: {e}")
                    return {}
        except Exception as e:
            logger.error(f"[GiftRaw]: Error in method GiftInfo(). Error: {e}")
            return {} if isinstance(slug, str) else []
    

    async def CollectionInfo(self, name: Union[str, List[str]]):
        try:
            if isinstance(name, list):
                results = []
                
                for i in range(0, len(name), self.batch_size):
                    batch = name[i:i + self.batch_size]
                    
                    tasks = []
                    for item in batch:
                        item = item.replace(" ", "").replace("'", "").replace("-", "")
                        tasks.append(self._make_request(url=f"https://app-api.xgift.tg/collections/{item}"))
                    
                    batch_responses = await asyncio.gather(*tasks, return_exceptions=True)
                    
                    for response in batch_responses:
                        if isinstance(response, Exception) or response is None:
                            results.append({})
                        else:
                            try:
                                results.append(response.json())
                            except Exception as e:
                                logger.error(f"[GiftRaw]: Error in method CollectionInfo(). Failed to parse JSON, Error: {e}")
                                results.append({})
                    
                    if i + self.batch_size < len(name) and self.delay > 0:
                        await asyncio.sleep(self.delay)
                
                return results

            else:
                name = name.replace(" ", "").replace("'", "").replace("-", "")
                response = await self._make_request(url=f"https://app-api.xgift.tg/collections/{name}")
                
                if response is None:
                    return {}
                
                try:
                    return response.json()
                except Exception as e:
                    logger.error(f"[GiftRaw]: Error in method CollectionInfo(). Failed to parse JSON, Error: {e}")
                    return {}
        except Exception as e:
            logger.error(f"[GiftRaw]: Error in method CollectionInfo(). Error: {e}")
            return {} if isinstance(name, str) else []
            

    async def CollectionGifts(self, name: Union[str, List[str]]):
        try:
            if isinstance(name, list):
                results = []
                
                for i in range(0, len(name), self.batch_size):
                    batch = name[i:i + self.batch_size]
                    
                    tasks = []
                    for item in batch:
                        item = item.replace(" ", "").replace("'", "").replace("-", "")
                        tasks.append(self._make_request(url=f"https://app-api.xgift.tg/gifts/filters/{item}", params={"collectionType": "upgradable"}))
                    
                    batch_responses = await asyncio.gather(*tasks, return_exceptions=True)
                    
                    for response in batch_responses:
                        if isinstance(response, Exception) or response is None:
                            results.append({})
                        else:
                            try:
                                results.append(response.json())
                            except Exception as e:
                                logger.error(f"[GiftRaw]: Error in method CollectionGifts(). Failed to parse JSON, Error: {e}")
                                results.append({})
                    
                    if i + self.batch_size < len(name) and self.delay > 0:
                        await asyncio.sleep(self.delay)
                
                return results

            else:
                name = name.replace(" ", "").replace("'", "").replace("-", "")
                response = await self._make_request(url=f"https://app-api.xgift.tg/gifts/filters/{name}", params={"collectionType": "upgradable"})
                
                if response is None:
                    return {}
                
                try:
                    return response.json()
                except Exception as e:
                    logger.error(f"[GiftRaw]: Error in method CollectionGifts(). Failed to parse JSON, Error: {e}")
                    return {}
        except Exception as e:
            logger.error(f"[GiftRaw]: Error in method CollectionGifts(). Error: {e}")
            return {} if isinstance(name, str) else []
            

    async def close(self):
        try:
            await self.client.close()
        except Exception as e:
            logger.error(f"[GiftRaw]: Error in method close(). Error: {e}")


from .raw import (
    GiftRaw
)
from .gift import (
    Gift
)
from .utils import (
    tonRate,
    nfts,
    lottie,
    emoji,
    graph
)

__all__ = [
    "GiftRaw",
    "Gift",
    "tonRate",
    "nfts",
    "lottie",
    "emoji",
    "graph"
]


from .raw import GiftRaw
from typing import *
import logging, json, time
from pathlib import Path

logger = logging.getLogger(__name__)

api = GiftRaw()

async def tonRate():
    try:
        response = await api._make_request("https://app-api.xgift.tg/utils/ton-rate")
        return response.json()
    except Exception as e:
        logger.error(f"[utils]: Error in method tonRate(). Error: {e}")
        return None

async def nfts(type: Literal["names", "ids", "all"]="all"):
    try:
        if type not in ["names", "ids", "all"]:
            error_msg = f"Unsupported type: {type}. Use 'names', 'ids' or 'all'"
            raise ValueError(error_msg)
            
        response = await api._make_request(f"https://api.changes.tg/names")

        if type == "names":
            response = list(response.json().keys())
        elif type == "ids":
            response = list(response.json().values())
        elif type == "all":
            response = response.json()

        return response
    except Exception as e:
        logger.error(f"[utils]: Error in method nfts(). Error: {e}")
        return None

async def lottie(slug: str):
    try:
        response = await api._make_request(f"https://nft.fragment.com/gift/{slug}.lottie.json")
        return response.json()
    except Exception as e:
        logger.error(f"[utils]: Error in method lottie(). Error: {e}")
        return None
    
async def emoji(collection_id="all"):
    current_dir = Path(__file__).parent
    json_file = current_dir / "gift_data.json"
    
    with open(json_file, "r", encoding="utf-8") as f:
        data = json.load(f)

    if collection_id == "all":
        return data
    elif collection_id in data:
        result = data[collection_id]
        return result
    else:
        print("[utils]: Try different id")

async def graph(collection_id):
    return f"https://static-gift.xgift.tg/gifts/graphs/{collection_id}.png?timestamp={int(time.time() * 1000)}"